#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/1/17 15:30
Desc: 东方财富-经济数据-澳大利亚
https://data.eastmoney.com/cjsj/foreign_5_0.html
"""

import pandas as pd
import requests


# 零售销售月率
def macro_australia_retail_rate_monthly() -> pd.DataFrame:
    """
    东方财富-经济数据-澳大利亚-零售销售月率
    https://data.eastmoney.com/cjsj/foreign_5_0.html
    :return: 零售销售月率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_AUSTRALIA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00152903")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"], errors="coerce").dt.date
    temp_df.sort_values(by="发布日期", ignore_index=True, inplace=True)
    return temp_df


# 贸易帐
def macro_australia_trade() -> pd.DataFrame:
    """
    东方财富-经济数据-澳大利亚-贸易帐
    https://data.eastmoney.com/cjsj/foreign_5_1.html
    :return: 贸易帐
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_AUSTRALIA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00152793")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"], errors="coerce").dt.date
    temp_df.sort_values(by="发布日期", ignore_index=True, inplace=True)
    return temp_df


# 失业率
def macro_australia_unemployment_rate() -> pd.DataFrame:
    """
    东方财富-经济数据-澳大利亚-失业率
    https://data.eastmoney.com/cjsj/foreign_5_2.html
    :return: 失业率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_AUSTRALIA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00101141")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"], errors="coerce").dt.date
    temp_df.sort_values(by="发布日期", ignore_index=True, inplace=True)
    return temp_df


# 生产者物价指数季率
def macro_australia_ppi_quarterly() -> pd.DataFrame:
    """
    东方财富-经济数据-澳大利亚-生产者物价指数季率
    https://data.eastmoney.com/cjsj/foreign_5_3.html
    :return: 生产者物价指数季率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_AUSTRALIA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00152722")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"], errors="coerce").dt.date
    temp_df.sort_values(by="发布日期", ignore_index=True, inplace=True)
    return temp_df


# 消费者物价指数季率
def macro_australia_cpi_quarterly() -> pd.DataFrame:
    """
    东方财富-经济数据-澳大利亚-消费者物价指数季率
    https://data.eastmoney.com/cjsj/foreign_5_4.html
    :return: 消费者物价指数季率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_AUSTRALIA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00101104")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"], errors="coerce").dt.date
    temp_df.sort_values(by="发布日期", ignore_index=True, inplace=True)
    return temp_df


# 消费者物价指数年率
def macro_australia_cpi_yearly() -> pd.DataFrame:
    """
    东方财富-经济数据-澳大利亚-消费者物价指数年率
    https://data.eastmoney.com/cjsj/foreign_5_5.html
    :return: 消费者物价指数年率
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_AUSTRALIA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00101093")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"], errors="coerce").dt.date
    temp_df.sort_values(by="发布日期", ignore_index=True, inplace=True)
    return temp_df


# 央行公布利率决议
def macro_australia_bank_rate() -> pd.DataFrame:
    """
    东方财富-经济数据-澳大利亚-央行公布利率决议
    https://data.eastmoney.com/cjsj/foreign_5_6.html
    :return: 央行公布利率决议
    :rtype: pandas.DataFrame
    """
    url = "https://datacenter-web.eastmoney.com/api/data/v1/get"
    params = {
        "reportName": "RPT_ECONOMICVALUE_AUSTRALIA",
        "columns": "ALL",
        "filter": '(INDICATOR_ID="EMG00342255")',
        "pageNumber": "1",
        "pageSize": "2000",
        "sortColumns": "REPORT_DATE",
        "sortTypes": "-1",
        "source": "WEB",
        "client": "WEB",
        "p": "1",
        "pageNo": "1",
        "pageNum": "1",
    }
    r = requests.get(url, params=params)
    data_json = r.json()
    temp_df = pd.DataFrame(data_json["result"]["data"])
    temp_df.columns = [
        "-",
        "-",
        "-",
        "时间",
        "-",
        "发布日期",
        "现值",
        "前值",
    ]
    temp_df = temp_df[
        [
            "时间",
            "前值",
            "现值",
            "发布日期",
        ]
    ]
    temp_df["前值"] = pd.to_numeric(temp_df["前值"], errors="coerce")
    temp_df["现值"] = pd.to_numeric(temp_df["现值"], errors="coerce")
    temp_df["发布日期"] = pd.to_datetime(temp_df["发布日期"], errors="coerce").dt.date
    temp_df.sort_values(by="发布日期", ignore_index=True, inplace=True)
    return temp_df


if __name__ == "__main__":
    macro_australia_retail_rate_monthly_df = macro_australia_retail_rate_monthly()
    print(macro_australia_retail_rate_monthly_df)

    macro_australia_trade_df = macro_australia_trade()
    print(macro_australia_trade_df)

    macro_australia_unemployment_rate_df = macro_australia_unemployment_rate()
    print(macro_australia_unemployment_rate_df)

    macro_australia_ppi_quarterly_df = macro_australia_ppi_quarterly()
    print(macro_australia_ppi_quarterly_df)

    macro_australia_cpi_quarterly_df = macro_australia_cpi_quarterly()
    print(macro_australia_cpi_quarterly_df)

    macro_australia_cpi_yearly_df = macro_australia_cpi_yearly()
    print(macro_australia_cpi_yearly_df)

    macro_australia_bank_rate_df = macro_australia_bank_rate()
    print(macro_australia_bank_rate_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/2/15 22:00
Desc: 东方财富-ETF行情
https://quote.eastmoney.com/sh513500.html
"""

from functools import lru_cache

import pandas as pd
import requests

from akshare.utils.func import fetch_paginated_data


@lru_cache()
def _fund_etf_code_id_map_em() -> dict:
    """
    东方财富-ETF代码和市场标识映射
    https://quote.eastmoney.com/center/gridlist.html#fund_etf
    :return: ETF 代码和市场标识映射
    :rtype: dict
    """
    url = "https://88.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "wbp2u": "|0|0|0|web",
        "fid": "f3",
        "fs": "b:MK0021,b:MK0022,b:MK0023,b:MK0024",
        "fields": "f3,f12,f13",
    }
    temp_df = fetch_paginated_data(url, params)
    temp_dict = dict(zip(temp_df["f12"], temp_df["f13"]))
    return temp_dict


def fund_etf_spot_em() -> pd.DataFrame:
    """
    东方财富-ETF 实时行情
    https://quote.eastmoney.com/center/gridlist.html#fund_etf
    :return: ETF 实时行情
    :rtype: pandas.DataFrame
    """
    url = "https://88.push2.eastmoney.com/api/qt/clist/get"
    params = {
        "pn": "1",
        "pz": "100",
        "po": "1",
        "np": "1",
        "ut": "bd1d9ddb04089700cf9c27f6f7426281",
        "fltt": "2",
        "invt": "2",
        "wbp2u": "|0|0|0|web",
        "fid": "f12",
        "fs": "b:MK0021,b:MK0022,b:MK0023,b:MK0024,b:MK0827",
        "fields": (
            "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,"
            "f12,f13,f14,f15,f16,f17,f18,f20,f21,"
            "f23,f24,f25,f22,f11,f30,f31,f32,f33,"
            "f34,f35,f38,f62,f63,f64,f65,f66,f69,"
            "f72,f75,f78,f81,f84,f87,f115,f124,f128,"
            "f136,f152,f184,f297,f402,f441"
        ),
    }
    temp_df = fetch_paginated_data(url, params)
    temp_df.rename(
        columns={
            "f12": "代码",
            "f14": "名称",
            "f2": "最新价",
            "f4": "涨跌额",
            "f3": "涨跌幅",
            "f5": "成交量",
            "f6": "成交额",
            "f7": "振幅",
            "f17": "开盘价",
            "f15": "最高价",
            "f16": "最低价",
            "f18": "昨收",
            "f8": "换手率",
            "f10": "量比",
            "f30": "现手",
            "f31": "买一",
            "f32": "卖一",
            "f33": "委比",
            "f34": "外盘",
            "f35": "内盘",
            "f62": "主力净流入-净额",
            "f184": "主力净流入-净占比",
            "f66": "超大单净流入-净额",
            "f69": "超大单净流入-净占比",
            "f72": "大单净流入-净额",
            "f75": "大单净流入-净占比",
            "f78": "中单净流入-净额",
            "f81": "中单净流入-净占比",
            "f84": "小单净流入-净额",
            "f87": "小单净流入-净占比",
            "f38": "最新份额",
            "f21": "流通市值",
            "f20": "总市值",
            "f402": "基金折价率",
            "f441": "IOPV实时估值",
            "f297": "数据日期",
            "f124": "更新时间",
        },
        inplace=True,
    )
    temp_df = temp_df[
        [
            "代码",
            "名称",
            "最新价",
            "IOPV实时估值",
            "基金折价率",
            "涨跌额",
            "涨跌幅",
            "成交量",
            "成交额",
            "开盘价",
            "最高价",
            "最低价",
            "昨收",
            "振幅",
            "换手率",
            "量比",
            "委比",
            "外盘",
            "内盘",
            "主力净流入-净额",
            "主力净流入-净占比",
            "超大单净流入-净额",
            "超大单净流入-净占比",
            "大单净流入-净额",
            "大单净流入-净占比",
            "中单净流入-净额",
            "中单净流入-净占比",
            "小单净流入-净额",
            "小单净流入-净占比",
            "现手",
            "买一",
            "卖一",
            "最新份额",
            "流通市值",
            "总市值",
            "数据日期",
            "更新时间",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["开盘价"] = pd.to_numeric(temp_df["开盘价"], errors="coerce")
    temp_df["最高价"] = pd.to_numeric(temp_df["最高价"], errors="coerce")
    temp_df["最低价"] = pd.to_numeric(temp_df["最低价"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    temp_df["量比"] = pd.to_numeric(temp_df["量比"], errors="coerce")
    temp_df["委比"] = pd.to_numeric(temp_df["委比"], errors="coerce")
    temp_df["外盘"] = pd.to_numeric(temp_df["外盘"], errors="coerce")
    temp_df["内盘"] = pd.to_numeric(temp_df["内盘"], errors="coerce")
    temp_df["流通市值"] = pd.to_numeric(temp_df["流通市值"], errors="coerce")
    temp_df["总市值"] = pd.to_numeric(temp_df["总市值"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["现手"] = pd.to_numeric(temp_df["现手"], errors="coerce")
    temp_df["买一"] = pd.to_numeric(temp_df["买一"], errors="coerce")
    temp_df["卖一"] = pd.to_numeric(temp_df["卖一"], errors="coerce")
    temp_df["最新份额"] = pd.to_numeric(temp_df["最新份额"], errors="coerce")
    temp_df["IOPV实时估值"] = pd.to_numeric(temp_df["IOPV实时估值"], errors="coerce")
    temp_df["基金折价率"] = pd.to_numeric(temp_df["基金折价率"], errors="coerce")
    temp_df["主力净流入-净额"] = pd.to_numeric(
        temp_df["主力净流入-净额"], errors="coerce"
    )
    temp_df["主力净流入-净占比"] = pd.to_numeric(
        temp_df["主力净流入-净占比"], errors="coerce"
    )
    temp_df["超大单净流入-净额"] = pd.to_numeric(
        temp_df["超大单净流入-净额"], errors="coerce"
    )
    temp_df["超大单净流入-净占比"] = pd.to_numeric(
        temp_df["超大单净流入-净占比"], errors="coerce"
    )
    temp_df["大单净流入-净额"] = pd.to_numeric(
        temp_df["大单净流入-净额"], errors="coerce"
    )
    temp_df["大单净流入-净占比"] = pd.to_numeric(
        temp_df["大单净流入-净占比"], errors="coerce"
    )
    temp_df["中单净流入-净额"] = pd.to_numeric(
        temp_df["中单净流入-净额"], errors="coerce"
    )
    temp_df["中单净流入-净占比"] = pd.to_numeric(
        temp_df["中单净流入-净占比"], errors="coerce"
    )
    temp_df["小单净流入-净额"] = pd.to_numeric(
        temp_df["小单净流入-净额"], errors="coerce"
    )
    temp_df["小单净流入-净占比"] = pd.to_numeric(
        temp_df["小单净流入-净占比"], errors="coerce"
    )
    temp_df["数据日期"] = pd.to_datetime(
        temp_df["数据日期"], format="%Y%m%d", errors="coerce"
    )
    temp_df["更新时间"] = (
        pd.to_datetime(temp_df["更新时间"], unit="s", errors="coerce")
        .dt.tz_localize("UTC")
        .dt.tz_convert("Asia/Shanghai")
    )
    return temp_df

def get_market_id(symbol: str)-> int:
    """
    东方财富-ETF市场标识判断
    :param symbol: ETF 代码
    :type symbol: str
    :return: ETF 代码和市场标识（1:上证 0:深证）
    :rtype: int 
    """
    if symbol.startswith(('0', '1', '3', '2', '5', '6')):
        if symbol.startswith(('5', '6')):
            return 1
        else:
            return 0
    else:
        return 1

def fund_etf_hist_em(
    symbol: str = "159707",
    period: str = "daily",
    start_date: str = "19700101",
    end_date: str = "20500101",
    adjust: str = "",
) -> pd.DataFrame:
    """
    东方财富-ETF行情
    https://quote.eastmoney.com/sz159707.html
    :param symbol: ETF 代码
    :type symbol: str
    :param period: choice of {'daily', 'weekly', 'monthly'}
    :type period: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :param adjust: choice of {"qfq": "前复权", "hfq": "后复权", "": "不复权"}
    :type adjust: str
    :return: 每日行情
    :rtype: pandas.DataFrame
    """
    # code_id_dict = _fund_etf_code_id_map_em()
    adjust_dict = {"qfq": "1", "hfq": "2", "": "0"}
    period_dict = {"daily": "101", "weekly": "102", "monthly": "103"}
    url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
    params = {
        "fields1": "f1,f2,f3,f4,f5,f6",
        "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61,f116",
        "ut": "7eea3edcaed734bea9cbfc24409ed989",
        "klt": period_dict[period],
        "fqt": adjust_dict[adjust],
        "beg": start_date,
        "end": end_date,
    }
    try:
        # market_id = code_id_dict[symbol]
        market_id = get_market_id(symbol)
        params.update({"secid": f"{market_id}.{symbol}"})
        r = requests.get(url, timeout=15, params=params)
        data_json = r.json()
    except KeyError:
        market_id = 1
        params.update({"secid": f"{market_id}.{symbol}"})
        r = requests.get(url, timeout=15, params=params)
        data_json = r.json()
        if not data_json["data"]:
            market_id = 0
            params.update({"secid": f"{market_id}.{symbol}"})
            r = requests.get(url, timeout=15, params=params)
            data_json = r.json()
    if not (data_json["data"] and data_json["data"]["klines"]):
        return pd.DataFrame()
    temp_df = pd.DataFrame([item.split(",") for item in data_json["data"]["klines"]])
    temp_df.columns = [
        "日期",
        "开盘",
        "收盘",
        "最高",
        "最低",
        "成交量",
        "成交额",
        "振幅",
        "涨跌幅",
        "涨跌额",
        "换手率",
    ]
    temp_df.index = pd.to_datetime(temp_df["日期"], errors="coerce")
    temp_df.reset_index(inplace=True, drop=True)
    temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
    temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
    return temp_df


def fund_etf_hist_min_em(
    symbol: str = "159707",
    start_date: str = "1979-09-01 09:32:00",
    end_date: str = "2222-01-01 09:32:00",
    period: str = "5",
    adjust: str = "",
) -> pd.DataFrame:
    """
    东方财富-ETF 行情
    https://quote.eastmoney.com/sz159707.html
    :param symbol: ETF 代码
    :type symbol: str
    :param start_date: 开始日期
    :type start_date: str
    :param end_date: 结束日期
    :type end_date: str
    :param period: choice of {"1", "5", "15", "30", "60"}
    :type period: str
    :param adjust: choice of {'', 'qfq', 'hfq'}
    :type adjust: str
    :return: 每日分时行情
    :rtype: pandas.DataFrame
    """
    #code_id_dict = _fund_etf_code_id_map_em()
    # 商品期货类 ETF
    # code_id_dict.update(
    #     {
    #         "159980": "0",
    #         "159981": "0",
    #         "159985": "0",
    #         "511090": "1",
    #         "511220": "1",
    #         "511380": "1",
    #     }
    # )
    adjust_map = {
        "": "0",
        "qfq": "1",
        "hfq": "2",
    }
    if period == "1":
        url = "https://push2his.eastmoney.com/api/qt/stock/trends2/get"
        params = {
            "fields1": "f1,f2,f3,f4,f5,f6,f7,f8,f9,f10,f11,f12,f13",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58",
            "ut": "7eea3edcaed734bea9cbfc24409ed989",
            "ndays": "5",
            "iscr": "0",
            "secid": f"{get_market_id(symbol)}.{symbol}",
        }
        r = requests.get(url, timeout=15, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["trends"]]
        )
        temp_df.columns = [
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "均价",
        ]
        temp_df.index = pd.to_datetime(temp_df["时间"])
        temp_df = temp_df[start_date:end_date]
        temp_df.reset_index(drop=True, inplace=True)
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["均价"] = pd.to_numeric(temp_df["均价"], errors="coerce")
        temp_df["时间"] = pd.to_datetime(temp_df["时间"]).astype(str)
        return temp_df
    else:
        url = "https://push2his.eastmoney.com/api/qt/stock/kline/get"
        params = {
            "fields1": "f1,f2,f3,f4,f5,f6",
            "fields2": "f51,f52,f53,f54,f55,f56,f57,f58,f59,f60,f61",
            "ut": "7eea3edcaed734bea9cbfc24409ed989",
            "klt": period,
            "fqt": adjust_map[adjust],
            "secid": f"{get_market_id(symbol)}.{symbol}",
            "beg": "0",
            "end": "20500000",
        }
        r = requests.get(url, timeout=15, params=params)
        data_json = r.json()
        temp_df = pd.DataFrame(
            [item.split(",") for item in data_json["data"]["klines"]]
        )
        temp_df.columns = [
            "时间",
            "开盘",
            "收盘",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "振幅",
            "涨跌幅",
            "涨跌额",
            "换手率",
        ]
        temp_df.index = pd.to_datetime(temp_df["时间"])
        temp_df = temp_df[start_date:end_date]
        temp_df.reset_index(drop=True, inplace=True)
        temp_df["开盘"] = pd.to_numeric(temp_df["开盘"], errors="coerce")
        temp_df["收盘"] = pd.to_numeric(temp_df["收盘"], errors="coerce")
        temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
        temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
        temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
        temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
        temp_df["振幅"] = pd.to_numeric(temp_df["振幅"], errors="coerce")
        temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
        temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
        temp_df["换手率"] = pd.to_numeric(temp_df["换手率"], errors="coerce")
        temp_df["时间"] = pd.to_datetime(temp_df["时间"]).astype(str)
        temp_df = temp_df[
            [
                "时间",
                "开盘",
                "收盘",
                "最高",
                "最低",
                "涨跌幅",
                "涨跌额",
                "成交量",
                "成交额",
                "振幅",
                "换手率",
            ]
        ]
        return temp_df


if __name__ == "__main__":
    fund_etf_spot_em_df = fund_etf_spot_em()
    print(fund_etf_spot_em_df)

    fund_etf_hist_hfq_em_df = fund_etf_hist_em(
        symbol="513500",
        period="daily",
        start_date="20000101",
        end_date="20230201",
        adjust="hfq",
    )
    print(fund_etf_hist_hfq_em_df)

    fund_etf_hist_qfq_em_df = fund_etf_hist_em(
        symbol="511010",
        period="daily",
        start_date="20000101",
        end_date="20230718",
        adjust="",
    )
    print(fund_etf_hist_qfq_em_df)

    fund_etf_hist_em_df = fund_etf_hist_em(
        symbol="159985",
        period="daily",
        start_date="20000101",
        end_date="20231211",
        adjust="",
    )
    print(fund_etf_hist_em_df)

    fund_etf_hist_min_em_df = fund_etf_hist_min_em(
        symbol="511380",
        period="1",
        adjust="",
        start_date="2025-03-10 09:30:00",
        end_date="2025-03-10 17:40:00",
    )
    print(fund_etf_hist_min_em_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/11/10 15:30
Desc: 新浪财经-基金行情
https://vip.stock.finance.sina.com.cn/fund_center/index.html#jjhqetf
"""

import pandas as pd
import py_mini_racer
import requests

from akshare.stock.cons import hk_js_decode
from akshare.utils import demjson


def fund_etf_category_sina(symbol: str = "LOF基金") -> pd.DataFrame:
    """
    新浪财经-基金列表
    https://vip.stock.finance.sina.com.cn/fund_center/index.html#jjhqetf
    :param symbol: choice of {"封闭式基金", "ETF基金", "LOF基金"}
    :type symbol: str
    :return: 指定 symbol 的基金列表
    :rtype: pandas.DataFrame
    """
    fund_map = {
        "封闭式基金": "close_fund",
        "ETF基金": "etf_hq_fund",
        "LOF基金": "lof_hq_fund",
    }
    url = (
        "https://vip.stock.finance.sina.com.cn/quotes_service/api/jsonp.php/"
        "IO.XSRV2.CallbackList['da_yPT46_Ll7K6WD']/Market_Center.getHQNodeDataSimple"
    )
    params = {
        "page": "1",
        "num": "5000",
        "sort": "symbol",
        "asc": "0",
        "node": fund_map[symbol],
        "[object HTMLDivElement]": "qvvne",
    }
    r = requests.get(url, params=params)
    data_text = r.text
    data_json = demjson.decode(data_text[data_text.find("([") + 1: -2])
    temp_df = pd.DataFrame(data_json)
    if symbol == "封闭式基金":
        temp_df.columns = [
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "买入",
            "卖出",
            "昨收",
            "今开",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "_",
            "_",
        ]
    else:
        temp_df.columns = [
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "买入",
            "卖出",
            "昨收",
            "今开",
            "最高",
            "最低",
            "成交量",
            "成交额",
            "_",
            "_",
            "_",
            "_",
        ]
    temp_df = temp_df[
        [
            "代码",
            "名称",
            "最新价",
            "涨跌额",
            "涨跌幅",
            "买入",
            "卖出",
            "昨收",
            "今开",
            "最高",
            "最低",
            "成交量",
            "成交额",
        ]
    ]
    temp_df["最新价"] = pd.to_numeric(temp_df["最新价"], errors="coerce")
    temp_df["涨跌额"] = pd.to_numeric(temp_df["涨跌额"], errors="coerce")
    temp_df["涨跌幅"] = pd.to_numeric(temp_df["涨跌幅"], errors="coerce")
    temp_df["买入"] = pd.to_numeric(temp_df["买入"], errors="coerce")
    temp_df["卖出"] = pd.to_numeric(temp_df["卖出"], errors="coerce")
    temp_df["昨收"] = pd.to_numeric(temp_df["昨收"], errors="coerce")
    temp_df["今开"] = pd.to_numeric(temp_df["今开"], errors="coerce")
    temp_df["最高"] = pd.to_numeric(temp_df["最高"], errors="coerce")
    temp_df["最低"] = pd.to_numeric(temp_df["最低"], errors="coerce")
    temp_df["成交量"] = pd.to_numeric(temp_df["成交量"], errors="coerce")
    temp_df["成交额"] = pd.to_numeric(temp_df["成交额"], errors="coerce")
    return temp_df


def fund_etf_hist_sina(symbol: str = "sh510050") -> pd.DataFrame:
    """
    新浪财经-基金-ETF 基金-日行情数据
    https://finance.sina.com.cn/fund/quotes/159996/bc.shtml
    :param symbol: 基金名称, 可以通过 ak.fund_etf_category_sina() 函数获取
    :type symbol: str
    :return: 日行情数据
    :rtype: pandas.DataFrame
    """
    url = f"https://finance.sina.com.cn/realstock/company/{symbol}/hisdata_klc2/klc_kl.js"
    r = requests.get(url)
    js_code = py_mini_racer.MiniRacer()
    js_code.eval(hk_js_decode)
    dict_list = js_code.call(
        "d", r.text.split("=")[1].split(";")[0].replace('"', "")
    )  # 执行js解密代码
    temp_df = pd.DataFrame(dict_list)
    if temp_df.empty:  # 处理获取数据为空的问题
        return pd.DataFrame()
    temp_df["date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.tz_localize(
        None
    )
    temp_df["open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df["high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df["low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df["close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    temp_df["volume"] = pd.to_numeric(temp_df["volume"], errors="coerce")

    # 转换日期列为日期类型
    temp_df["date"] = temp_df["date"].dt.date
    temp_df = temp_df.sort_values(by="date", ascending=True)
    return temp_df


def fund_etf_dividend_sina(symbol: str = "sh510050") -> pd.DataFrame:
    """
    新浪财经-基金-ETF 基金-累计分红
    https://finance.sina.com.cn/fund/quotes/510050/bc.shtml
    :param symbol: 基金名称, 可以通过 ak.fund_etf_category_sina() 函数获取
    :type symbol: str
    :return: 累计分红
    :rtype: pandas.DataFrame
    """
    # 构建复权数据URL
    factor_url = f"https://finance.sina.com.cn/realstock/company/{symbol}/hfq.js"
    r = requests.get(factor_url)
    text = r.text
    if text.startswith("var"):
        json_str = text.split("=")[1].strip().rsplit("}", maxsplit=1)[0].strip()
        data = eval(json_str + "}")  # 这里使用eval而不是json.loads因为数据格式特殊

        if isinstance(data, dict) and "data" in data:
            df = pd.DataFrame(data["data"])
            # 重命名列
            df.columns = ["date", "f", "s", "u"] if len(df.columns) == 4 else df.columns
            # 移除1900-01-01的数据
            df = df[df["date"] != "1900-01-01"]
            # 转换日期
            df["date"] = pd.to_datetime(df["date"])
            # 转换数值类型
            df[["f", "s", "u"]] = df[["f", "s", "u"]].astype(float)
            # 按日期排序
            df = df.sort_values(by="date", ascending=True, ignore_index=True)
            temp_df = df[["date", "u"]].copy()
            temp_df.columns = ["日期", "累计分红"]
            temp_df["日期"] = pd.to_datetime(temp_df["日期"], errors="coerce").dt.date
            return temp_df
        else:
            return pd.DataFrame()
    else:
        return pd.DataFrame()


if __name__ == "__main__":
    fund_etf_category_sina_df = fund_etf_category_sina(symbol="封闭式基金")
    print(fund_etf_category_sina_df)

    fund_etf_category_sina_df = fund_etf_category_sina(symbol="ETF基金")
    print(fund_etf_category_sina_df)

    fund_etf_category_sina_df = fund_etf_category_sina(symbol="LOF基金")
    print(fund_etf_category_sina_df)

    fund_etf_hist_sina_df = fund_etf_hist_sina(symbol="sh510050")
    print(fund_etf_hist_sina_df)

    fund_etf_dividend_sina_df = fund_etf_dividend_sina(symbol="sh510050")
    print(fund_etf_dividend_sina_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2021/5/27 20:19
Desc: 指数配置文件
"""

# weibo-user-agent
index_weibo_headers = {
    "User-Agent": "Mozilla/5.0 (iPhone; CPU iPhone OS 11_0 like Mac OS X) "
    "AppleWebKit/604.1.38 (KHTML, like Gecko) Version/11.0 Mobile/15A372 Safari/604.1",
    "Referer": "http://data.weibo.com/index/newindex",
    "Accept": "application/json",
    "Origin": "https://data.weibo.com",
}

# sw-cons
sw_cons_headers = {
    "Accept": "*/*",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Connection": "keep-alive",
    "Content-Length": "34",
    "Content-Type": "text/plain; charset=UTF-8",
    # "Cookie": "ASP.NET_SessionId=i55eaz55142xdxfx0bkqp145",
    "Host": "www.swsindex.com",
    "Origin": "http://www.swsindex.com",
    "Referer": "http://www.swsindex.com/idx0210.aspx?swindexcode=801010",
    "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/78.0.3904.108 Safari/537.36",
    "X-AjaxPro-Method": "ReturnContent",
}

# sw-url
sw_url = "http://www.swsindex.com/handler.aspx"

# sw-payload
sw_payload = {
    "tablename": "swzs",
    "key": "L1",
    "p": "1",
    "where": "L1 in('801010','801020','801030','801040','801050','801080','801110','801120','801130',"
    "'801140','801150','801160','801170','801180','801200','801210','801230','801710','801720',"
    "'801730','801740','801750','801760','801770','801780','801790','801880','801890','801950',"
    "'801960','801970','801980')",
    "orderby": "",
    "fieldlist": "L1,L2,L3,L4,L5,L6,L7,L8,L11",
    "pagecount": "28",
    "timed": "",
}

# sw-headers
sw_headers = {
    "Accept": "application/json, text/javascript, */*",
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
    "Cache-Control": "no-cache",
    "Content-Type": "application/x-www-form-urlencoded",
    "DNT": "1",
    "Host": "www.swsindex.com",
    "Origin": "http://www.swsindex.com",
    "Pragma": "no-cache",
    "Referer": "http://www.swsindex.com/idx0120.aspx?columnid=8832",
    "User-Agent": "Mozilla/5.0 (Windows NT 6.1; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/74.0.3729.169 Safari/537.36",
    "X-Requested-With": "XMLHttpRequest",
}

# zh-sina-a
zh_sina_index_stock_url = (
    "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/"
    "Market_Center.getHQNodeDataSimple"
)
zh_sina_index_stock_payload = {
    "page": "1",
    "num": "80",
    "sort": "symbol",
    "asc": "1",
    "node": "hs_s",
    "_s_r_a": "page",
}
zh_sina_index_stock_count_url = (
    "http://vip.stock.finance.sina.com.cn/quotes_service/api/json_v2.php/"
    "Market_Center.getHQNodeStockCountSimple?node=hs_s"
)
zh_sina_index_stock_hist_url = (
    "https://finance.sina.com.cn/realstock/company/{}/hisdata/klc_kl.js"
)

# investing
short_headers = {
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/61.0.3163.91 Safari/537.36"
}

long_headers = {
    "accept": "text/plain, */*; q=0.01",
    # 'accept-encoding': 'gzip, deflate, br',
    "accept-language": "zh-CN,zh;q=0.9,en;q=0.8",
    # 'cache-control': 'no-cache',
    "content-length": "267",
    "content-type": "application/x-www-form-urlencoded",
    "origin": "https://cn.investing.com",
    "referer": "https://cn.investing.com/commodities/brent-oil-historical-data",
    "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
    "Chrome/61.0.3163.91 Safari/537.36",
    "x-requested-with": "XMLHttpRequest",
}

index_global_sina_symbol_map = {
    # 欧洲股市
    "英国富时100指数": "UKX",
    "德国DAX 30种股价指数": "DAX",
    "俄罗斯MICEX指数": "INDEXCF",
    "法CAC40指数": "CAC",
    "瑞士股票指数": "SWI20",
    "富时意大利MIB指数": "FTSEMIB",
    "荷兰AEX综合指数": "AEX",
    "西班牙IBEX指数": "IBEX",
    "欧洲Stoxx50指数": "SX5E",
    # 美洲股市
    "加拿大S&P/TSX综合指数": "GSPTSE",
    "墨西哥BOLSA指数": "MXX",
    "巴西BOVESPA股票指数": "IBOV",
    # 亚洲股市
    "中国台湾加权指数": "TWJQ",
    "日经225指数": "NKY",
    "首尔综合指数": "KOSPI",
    "印度尼西亚雅加达综合指数": "JCI",
    "印度孟买SENSEX指数": "SENSEX",
    # 澳洲股市
    "澳大利亚标准普尔200指数": "AS51",
    "新西兰NZSE 50指数": "NZ250",
    # 非洲股市
    "埃及CASE 30指数": "CASE",
}

index_global_em_symbol_map = {
    "波罗的海BDI指数": {"code": "BDI", "market": "100"},
    "葡萄牙PSI20": {"code": "PSI20", "market": "100"},
    "菲律宾马尼拉": {"code": "PSI", "market": "100"},
    "泰国SET": {"code": "SET", "market": "100"},
    "俄罗斯RTS": {"code": "RTS", "market": "100"},
    "巴基斯坦卡拉奇": {"code": "KSE100", "market": "100"},
    "越南胡志明": {"code": "VNINDEX", "market": "100"},
    "红筹指数": {"code": "HSCCI", "market": "124"},
    "印尼雅加达综合": {"code": "JKSE", "market": "100"},
    "希腊雅典ASE": {"code": "ASE", "market": "100"},
    "墨西哥BOLSA": {"code": "MXX", "market": "100"},
    "挪威OSEBX": {"code": "OSEBX", "market": "100"},
    "巴西BOVESPA": {"code": "BVSP", "market": "100"},
    "波兰WIG": {"code": "WIG", "market": "100"},
    "印度孟买SENSEX": {"code": "SENSEX", "market": "100"},
    "布拉格指数": {"code": "PX", "market": "100"},
    "荷兰AEX": {"code": "AEX", "market": "100"},
    "冰岛ICEX": {"code": "ICEXI", "market": "100"},
    "斯里兰卡科伦坡": {"code": "CSEALL", "market": "100"},
    "富时新加坡海峡时报": {"code": "STI", "market": "100"},
    "富时意大利MIB": {"code": "MIB", "market": "100"},
    "路透CRB商品指数": {"code": "CRB", "market": "100"},
    "比利时BFX": {"code": "BFX", "market": "100"},
    "富时AIM全股": {"code": "AXX", "market": "100"},
    "新西兰50": {"code": "NZ50", "market": "100"},
    "上证指数": {"code": "000001", "market": "1"},
    "国企指数": {"code": "HSCEI", "market": "100"},
    "沪深300": {"code": "000300", "market": "1"},
    "英国富时100": {"code": "FTSE", "market": "100"},
    "中小100": {"code": "399005", "market": "0"},
    "瑞士SMI": {"code": "SSMI", "market": "100"},
    "西班牙IBEX35": {"code": "IBEX", "market": "100"},
    "瑞典OMXSPI": {"code": "OMXSPI", "market": "100"},
    "爱尔兰综合": {"code": "ISEQ", "market": "100"},
    "韩国KOSPI": {"code": "KS11", "market": "100"},
    "深证成指": {"code": "399001", "market": "0"},
    "韩国KOSPI200": {"code": "KOSPI200", "market": "100"},
    "芬兰赫尔辛基": {"code": "HEX", "market": "100"},
    "恒生指数": {"code": "HSI", "market": "100"},
    "欧洲斯托克50": {"code": "SX5E", "market": "100"},
    "美元指数": {"code": "UDI", "market": "100"},
    "法国CAC40": {"code": "FCHI", "market": "100"},
    "台湾加权": {"code": "TWII", "market": "100"},
    "英国富时250": {"code": "MCX", "market": "100"},
    "富时马来西亚KLCI": {"code": "KLSE", "market": "100"},
    "OMX哥本哈根20": {"code": "OMXC20", "market": "100"},
    "道琼斯": {"code": "DJIA", "market": "100"},
    "奥地利ATX": {"code": "ATX", "market": "100"},
    "加拿大S&P/TSX": {"code": "TSX", "market": "100"},
    "德国DAX30": {"code": "GDAXI", "market": "100"},
    "创业板指": {"code": "399006", "market": "0"},
    "澳大利亚普通股": {"code": "AORD", "market": "100"},
    "标普500": {"code": "SPX", "market": "100"},
    "澳大利亚标普200": {"code": "AS51", "market": "100"},
    "日经225": {"code": "N225", "market": "100"},
    "纳斯达克": {"code": "NDX", "market": "100"},
}



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2025/12/22 17:00
Desc: 天天基金-基金档案
https://fundf10.eastmoney.com/jjfl_015641.html
"""
import re
from io import StringIO

import pandas as pd
import requests
from bs4 import BeautifulSoup


def fund_fee_em(symbol: str = "015641", indicator: str = "认购费率") -> pd.DataFrame:
    """
    天天基金-基金档案-购买信息
    https://fundf10.eastmoney.com/jjfl_015641.html
    :param symbol: 基金代码
    :type symbol: str
    :param indicator: choice of {"交易状态", "申购与赎回金额", "交易确认日", "运作费用", "认购费率", "申购费率", "赎回费率"}
    :type indicator: str
    :return: 交易规则
    :rtype: pandas.DataFrame
    """
    url = f"https://fundf10.eastmoney.com/jjfl_{symbol}.html"
    r = requests.get(url)
    # 使用BeautifulSoup解析HTML
    soup = BeautifulSoup(r.text, "html.parser")
    # 创建一个字典，将标题文本映射到对应的表格
    tables_dict = {}
    # 找到所有具有class="t"的h4标签
    title_elements = soup.find_all(name="h4", class_="t")
    for title_elem in title_elements:
        # 获取标题文本
        title_text = title_elem.get_text(strip=True)
        title_text = re.sub(r'\s+', ' ', title_text).strip()

        # 获取标题后面的表格
        if title_text == "申购与赎回金额":
            next_table = title_elem.find_all_next("table")[0]
            next_next_table = title_elem.find_all_next("table")[1]
            table_html = str(next_table)
            next_table_html = str(next_next_table)
            df_1 = pd.read_html(StringIO(table_html))[0]
            df_2 = pd.read_html(StringIO(next_table_html))[0]
            df = pd.concat(objs=[df_1, df_2], ignore_index=True)
            tables_dict[title_text] = df
            continue
        else:
            next_table = title_elem.find_next("table")

        if next_table:
            try:
                # 将表格转换为HTML字符串，然后使用pd.read_html读取
                table_html = str(next_table)
                df = pd.read_html(StringIO(table_html))[0]
                tables_dict[title_text] = df
            except Exception as e:
                continue

    if indicator == "交易状态":
        temp_df = tables_dict[indicator]
    elif indicator == "申购与赎回金额":
        temp_df = tables_dict[indicator]
    elif indicator == "交易确认日":
        temp_df = tables_dict[indicator]
    elif indicator == "运作费用":
        temp_df = tables_dict[indicator]
    elif indicator == "认购费率（前端）":
        temp_df = tables_dict[indicator]
        temp_df["原费率"] = temp_df["原费率|天天基金优惠费率"].str.split(
            "|", expand=True
        )[0]
        temp_df["天天基金优惠费率"] = temp_df["原费率|天天基金优惠费率"].str.split(
            "|", expand=True
        )[1]
        del temp_df["原费率|天天基金优惠费率"]
        temp_df.loc[3, "天天基金优惠费率"] = temp_df.loc[3, "原费率"]
        temp_df["原费率"] = temp_df["原费率"].str.strip()
        temp_df["天天基金优惠费率"] = temp_df["天天基金优惠费率"].str.strip()
    elif indicator == "申购费率（前端）":
        temp_df = tables_dict[indicator]
        if temp_df[
            "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
        ].str.split("|", expand=True).shape == (1, 1):
            temp_df["原费率"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[0]
            temp_df["天天基金优惠费率-银行卡购买"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[0]
            temp_df["天天基金优惠费率-活期宝购买"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[0]
            del temp_df["原费率|天天基金优惠费率 银行卡购买|活期宝购买"]
        elif temp_df[
            "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
        ].str.split("|", expand=True).shape == (3, 3):
            temp_df["原费率"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[0]
            temp_df["天天基金优惠费率-银行卡购买"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[1]
            temp_df["天天基金优惠费率-活期宝购买"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[2]
            temp_df.loc[2, "天天基金优惠费率-银行卡购买"] = temp_df.loc[2, "原费率"]
            temp_df.loc[2, "天天基金优惠费率-活期宝购买"] = temp_df.loc[2, "原费率"]
            del temp_df["原费率|天天基金优惠费率 银行卡购买|活期宝购买"]
        else:
            temp_df["原费率"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[0]
            temp_df["天天基金优惠费率-银行卡购买"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[1]
            temp_df["天天基金优惠费率-活期宝购买"] = temp_df[
                "原费率|天天基金优惠费率 银行卡购买|活期宝购买"
            ].str.split("|", expand=True)[2]
            del temp_df["原费率|天天基金优惠费率 银行卡购买|活期宝购买"]
            temp_df.loc[3, "天天基金优惠费率-银行卡购买"] = temp_df.loc[3, "原费率"]
            temp_df.loc[3, "天天基金优惠费率-活期宝购买"] = temp_df.loc[3, "原费率"]
        temp_df["原费率"] = temp_df["原费率"].str.strip()
        temp_df["天天基金优惠费率-银行卡购买"] = temp_df[
            "天天基金优惠费率-银行卡购买"
        ].str.strip()
        temp_df["天天基金优惠费率-活期宝购买"] = temp_df[
            "天天基金优惠费率-活期宝购买"
        ].str.strip()
    elif indicator == "赎回费率":
        temp_df = tables_dict[indicator]
    else:
        temp_df = pd.DataFrame([])
    return temp_df


if __name__ == "__main__":
    fund_fee_em_df = fund_fee_em(symbol="019005", indicator="交易状态")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="019005", indicator="申购与赎回金额")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="019005", indicator="交易确认日")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="019005", indicator="运作费用")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="019005", indicator="认购费率（前端）")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="019005", indicator="申购费率（前端）")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="019005", indicator="赎回费率")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="022364", indicator="申购费率（前端）")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="022365", indicator="申购费率（前端）")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="006030", indicator="申购费率（前端）")
    print(fund_fee_em_df)

    fund_fee_em_df = fund_fee_em(symbol="016243", indicator="申购费率（前端）")
    print(fund_fee_em_df)


#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/2/20 14:00
Desc: 电影票房数据
https://ys.endata.cn/BoxOffice/Movie
"""

import datetime
import json
import os

import pandas as pd
import requests
import py_mini_racer


def _get_js_path(name: str = "", module_file: str = "") -> str:
    """
    get JS file path
    :param name: file name
    :type name: str
    :param module_file: filename
    :type module_file: str
    :return: 路径
    :rtype: str
    """
    module_folder = os.path.abspath(os.path.dirname(os.path.dirname(module_file)))
    module_json_path = os.path.join(module_folder, "movie", name)
    return module_json_path


def _get_file_content(file_name: str = "jm.js"):
    """
    read the file content
    :param file_name: filename
    :type file_name: str
    :return: file content
    :rtype: str
    """
    setting_file_name = file_name
    setting_file_path = _get_js_path(setting_file_name, __file__)
    with open(setting_file_path) as f:
        file_data = f.read()
    return file_data


def get_current_week(date: str = "20201019") -> datetime.date:
    """
    当前周的周一
    :param date: 具体的日期
    :type date: str
    :return: 当前周的周一
    :rtype: datetime.date
    """
    monday = datetime.datetime.strptime(date, "%Y%m%d").date()
    one_day = datetime.timedelta(days=1)
    while monday.weekday() != 0:
        monday -= one_day
    return monday


def decrypt(origin_data: str = "") -> str:
    """
    解密
    :param origin_data: 解密前的字符串
    :type origin_data: str
    :return: 解密后的字符串
    :rtype: str
    """
    file_data = _get_file_content(file_name="jm.js")
    ctx = py_mini_racer.MiniRacer()
    ctx.eval(file_data)
    data = ctx.call("webInstace.shell", origin_data)
    return data


def movie_boxoffice_realtime() -> pd.DataFrame:
    """
    电影票房-实时票房
    https://ys.endata.cn/BoxOffice/Movie
    :return: 实时票房数据
    :rtype: pandas.DataFrame
    """
    today = datetime.datetime.today().date().strftime("%Y%m%d")
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {
        "showDate": "",
        "tdate": f"{today[:4]}-{today[4:6]}-{today[6:]}",
        "MethodName": "BoxOffice_GetHourBoxOffice",
    }
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table1"])
    temp_df = temp_df.iloc[:, :7]
    temp_df.columns = [
        "排序",
        "_",
        "影片名称",
        "实时票房",
        "累计票房",
        "上映天数",
        "票房占比",
    ]
    temp_df = temp_df[
        ["排序", "影片名称", "实时票房", "票房占比", "上映天数", "累计票房"]
    ]
    return temp_df


def movie_boxoffice_daily(date: str = "20240219") -> pd.DataFrame:
    """
    电影票房-单日票房
    https://www.endata.com.cn/BoxOffice/BO/Day/index.html
    :param date: 只能设置当前日期的前一天的票房数据
    :type date: str
    :return: 每日票房
    :rtype: pandas.DataFrame
    """
    last_date = datetime.datetime.strptime(date, "%Y%m%d") - datetime.timedelta(days=1)
    last_date = last_date.strftime("%Y%m%d")
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {
        "sdate": f"{date[:4]}-{date[4:6]}-{date[6:]}",
        "edate": f"{last_date[:4]}-{last_date[4:6]}-{last_date[6:]}",
        "MethodName": "BoxOffice_GetDayBoxOffice",
    }
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    temp_df.columns = [
        "排序",
        "_",
        "影片名称",
        "_",
        "累计票房",
        "平均票价",
        "上映天数",
        "场均人次",
        "_",
        "_",
        "_",
        "_",
        "_",
        "单日票房",
        "环比变化",
        "_",
        "口碑指数",
    ]
    temp_df = temp_df[
        [
            "排序",
            "影片名称",
            "单日票房",
            "环比变化",
            "累计票房",
            "平均票价",
            "场均人次",
            "口碑指数",
            "上映天数",
        ]
    ]
    return temp_df


def movie_boxoffice_weekly(date: str = "20240218") -> pd.DataFrame:
    """
    电影票房-单周票房
    https://www.endata.com.cn/BoxOffice/BO/Week/oneWeek.html
    :param date: 只能获取指定日期所在完整周的票房数据
    :type date: str
    :return: 单周票房
    :rtype: pandas.DataFrame
    """
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {
        "sdate": get_current_week(date=date).strftime("%Y-%m-%d"),
        "MethodName": "BoxOffice_GetWeekInfoData",
    }
    r = requests.post(url, data=payload)
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    temp_df.columns = [
        "排序",
        "_",
        "影片名称",
        "单周票房",
        "累计票房",
        "_",
        "上映天数",
        "平均票价",
        "场均人次",
        "环比变化",
        "_",
        "_",
        "_",
        "排名变化",
        "口碑指数",
    ]
    temp_df = temp_df[
        [
            "排序",
            "影片名称",
            "排名变化",
            "单周票房",
            "环比变化",
            "累计票房",
            "平均票价",
            "场均人次",
            "口碑指数",
            "上映天数",
        ]
    ]
    temp_df["单周票房"] = pd.to_numeric(temp_df["单周票房"], errors="coerce")
    temp_df["环比变化"] = pd.to_numeric(temp_df["环比变化"], errors="coerce")
    temp_df["累计票房"] = pd.to_numeric(temp_df["累计票房"], errors="coerce")
    return temp_df


def movie_boxoffice_monthly(date: str = "20240218") -> pd.DataFrame:
    """
    电影票房-单月票房
    https://www.endata.com.cn/BoxOffice/BO/Month/oneMonth.html
    :param date: 指定日期所在月份的月度票房
    :type date: str
    :return: 单月票房
    :rtype: pandas.DataFrame
    """
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {
        "startTime": f"{date[:4]}-{date[4:6]}-01",
        "MethodName": "BoxOffice_GetMonthBox",
    }
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    temp_df.columns = [
        "排序",
        "_",
        "影片名称",
        "月内天数",
        "单月票房",
        "平均票价",
        "场均人次",
        "月度占比",
        "上映日期",
        "_",
        "口碑指数",
    ]
    temp_df = temp_df[
        [
            "排序",
            "影片名称",
            "单月票房",
            "月度占比",
            "平均票价",
            "场均人次",
            "上映日期",
            "口碑指数",
            "月内天数",
        ]
    ]
    temp_df["上映日期"] = pd.to_datetime(temp_df["上映日期"], errors="coerce").dt.date
    return temp_df


def movie_boxoffice_yearly(date: str = "20240218") -> pd.DataFrame:
    """
    电影票房-年度票房
    https://www.endata.com.cn/BoxOffice/BO/Year/index.html
    :param date: 当前日期所在年度的票房数据
    :type date: str
    :return: 年度票房
    :rtype: pandas.DataFrame
    """
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {
        "year": f"{date[:4]}",
        "MethodName": "BoxOffice_GetYearInfoData",
    }
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    temp_df.reset_index(inplace=True)
    temp_df.columns = [
        "排序",
        "_",
        "影片名称",
        "类型",
        "总票房",
        "平均票价",
        "场均人次",
        "国家及地区",
        "上映日期",
        "_",
    ]
    temp_df["排序"] = range(1, len(temp_df) + 1)
    temp_df = temp_df[
        [
            "排序",
            "影片名称",
            "类型",
            "总票房",
            "平均票价",
            "场均人次",
            "国家及地区",
            "上映日期",
        ]
    ]
    temp_df["上映日期"] = pd.to_datetime(temp_df["上映日期"], errors="coerce").dt.date
    return temp_df


def movie_boxoffice_yearly_first_week(date: str = "20201018") -> pd.DataFrame:
    """
    电影票房-年度票房-年度首周票房
    https://www.endata.com.cn/BoxOffice/BO/Year/firstWeek.html
    :param date: 当前日期所在年度的年度首周票房票房数据
    :type date: str
    :return: 年度首周票房
    :rtype: pandas.DataFrame
    """
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {
        "year": f"{date[:4]}",
        "MethodName": "BoxOffice_getYearInfo_fData",
    }
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    temp_df.reset_index(inplace=True)
    temp_df.columns = [
        "排序",
        "_",
        "_",
        "影片名称",
        "首周票房",
        "场均人次",
        "上映日期",
        "首周天数",
        "类型",
        "国家及地区",
        "_",
        "占总票房比重",
    ]
    temp_df["排序"] = range(1, len(temp_df) + 1)
    temp_df = temp_df[
        [
            "排序",
            "影片名称",
            "类型",
            "首周票房",
            "占总票房比重",
            "场均人次",
            "国家及地区",
            "上映日期",
            "首周天数",
        ]
    ]
    temp_df["上映日期"] = pd.to_datetime(temp_df["上映日期"], errors="coerce").dt.date
    return temp_df


def movie_boxoffice_cinema_daily(date: str = "20240219") -> pd.DataFrame:
    """
    电影票房-影院票房-日票房排行
    https://www.endata.com.cn/BoxOffice/BO/Cinema/day.html
    :param date: 当前日期前一日的票房数据
    :type date: str
    :return: 影票房-影院票房-日票房排行
    :rtype: pandas.DataFrame
    """
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {
        "rowNum1": "1",
        "rowNum2": "100",
        "date": date,
        "MethodName": "BoxOffice_GetCinemaDayBoxOffice",
    }
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    temp_df.columns = [
        "排序",
        "_",
        "影院名称",
        "单日票房",
        "单日场次",
        "_",
        "_",
        "场均票价",
        "场均人次",
        "上座率",
    ]
    temp_df = temp_df[
        ["排序", "影院名称", "单日票房", "单日场次", "场均人次", "场均票价", "上座率"]
    ]
    return temp_df


def movie_boxoffice_cinema_weekly(date: str = "20240219") -> pd.DataFrame:
    """
    电影票房-影院票房-周票房排行
    https://www.endata.com.cn/BoxOffice/BO/Cinema/week.html
    :param date: 当前日期前完整一周的票房数据
    :type date: str
    :return: 影票房-影院票房-轴票房排行
    :rtype: pandas.DataFrame
    """
    url = "https://www.endata.com.cn/API/GetData.ashx"
    payload = {
        "dateID": str(
            datetime.date.fromisoformat(
                f"{date[:4]}-{date[4:6]}-{date[6:]}"
            ).isocalendar()[1]
            - 1
            - 41
            + 1128
        ),
        "rowNum1": "1",
        "rowNum2": "100",
        "MethodName": "BoxOffice_GetCinemaWeekBoxOffice",
    }
    r = requests.post(url, data=payload)
    r.encoding = "utf8"
    data_json = json.loads(decrypt(r.text))
    temp_df = pd.DataFrame(data_json["Data"]["Table"])
    temp_df.columns = [
        "排序",
        "_",
        "影院名称",
        "当周票房",
        "_",
        "单银幕票房",
        "场均人次",
        "单日单厅票房",
        "单日单厅场次",
    ]
    temp_df = temp_df[
        [
            "排序",
            "影院名称",
            "当周票房",
            "单银幕票房",
            "场均人次",
            "单日单厅票房",
            "单日单厅场次",
        ]
    ]
    return temp_df


if __name__ == "__main__":
    movie_boxoffice_realtime_df = movie_boxoffice_realtime()
    print(movie_boxoffice_realtime_df)

    movie_boxoffice_daily_df = movie_boxoffice_daily(date="20240219")
    print(movie_boxoffice_daily_df)

    movie_boxoffice_weekly_df = movie_boxoffice_weekly(date="20240218")
    print(movie_boxoffice_weekly_df)

    movie_boxoffice_monthly_df = movie_boxoffice_monthly(date="20240218")
    print(movie_boxoffice_monthly_df)

    movie_boxoffice_yearly_df = movie_boxoffice_yearly(date="20240218")
    print(movie_boxoffice_yearly_df)

    movie_boxoffice_yearly_first_week_df = movie_boxoffice_yearly_first_week(
        date="20201018"
    )
    print(movie_boxoffice_yearly_first_week_df)

    movie_boxoffice_cinema_daily_df = movie_boxoffice_cinema_daily(date="20240219")
    print(movie_boxoffice_cinema_daily_df)

    movie_boxoffice_cinema_weekly_df = movie_boxoffice_cinema_weekly(date="20240219")
    print(movie_boxoffice_cinema_weekly_df)



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""
Date: 2024/4/25 17:00
Desc: 新闻联播文字稿
https://tv.cctv.com/lm/xwlb
"""

import re

import pandas as pd
import requests
from bs4 import BeautifulSoup
from tqdm import tqdm


def news_cctv(date: str = "20240424") -> pd.DataFrame:
    """
    新闻联播文字稿
    https://tv.cctv.com/lm/xwlb
    :param date: 需要获取数据的日期; 目前 20160203 年后
    :type date: str
    :return: 新闻联播文字稿
    :rtype: pandas.DataFrame
    """
    if int(date) <= int("20130708"):
        url = f"https://cctv.cntv.cn/lm/xinwenlianbo/{date}.shtml"
        r = requests.get(url)
        r.encoding = "gbk"
        raw_list = re.findall(r"title_array_01\((.*)", r.text)
        page_url = [
            re.findall("(http.*)", item)[0].split("'")[0] for item in raw_list[1:]
        ]
        title_list = []
        content_list = []
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,"
            "image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "Accept-Encoding": "gzip, deflate",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Cookie": "cna=DLYSGBDthG4CAbRVCNxSxGT6",
            "Host": "tv.cctv.com",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/92.0.4515.159 Safari/537.36",
        }
        for page in tqdm(page_url, leave=False):
            try:
                r = requests.get(page, headers=headers)
                r.encoding = "utf-8"
                soup = BeautifulSoup(r.text, "lxml")
                title = soup.find("h3").text
                content = soup.find("div", attrs={"class": "cnt_bd"}).text
                title_list.append(title.strip("[视频]").strip().replace("\n", " "))
                content_list.append(
                    content.strip()
                    .strip("央视网消息(新闻联播)：")
                    .strip("央视网消息（新闻联播）：")
                    .strip("(新闻联播)：")
                    .strip()
                    .replace("\n", " ")
                )
            except:  # noqa: E722
                continue
        temp_df = pd.DataFrame(
            data=[[date] * len(title_list), title_list, content_list],
            index=["date", "title", "content"],
        ).T
        return temp_df

    elif int(date) < int("20160203"):
        url = f"https://cctv.cntv.cn/lm/xinwenlianbo/{date}.shtml"
        r = requests.get(url)
        r.encoding = "utf-8"
        soup = BeautifulSoup(r.text, "lxml")
        page_url = [
            item.find("a")["href"]
            for item in soup.find(
                name="div", attrs={"id": "contentELMT1368521805488378"}
            ).find_all("li")[1:]
        ]
        title_list = []
        content_list = []
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,"
            "image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "Accept-Encoding": "gzip, deflate",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Cookie": "cna=DLYSGBDthG4CAbRVCNxSxGT6",
            "Host": "tv.cctv.com",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/92.0.4515.159 Safari/537.36",
        }
        for page in tqdm(page_url, leave=False):
            try:
                r = requests.get(page, headers=headers)
                r.encoding = "utf-8"
                soup = BeautifulSoup(r.text, features="lxml")
                title = soup.find("h3").text
                content = soup.find(name="div", attrs={"class": "cnt_bd"}).text
                title_list.append(title.strip("[视频]").strip().replace("\n", " "))
                content_list.append(
                    content.strip()
                    .strip("央视网消息(新闻联播)：")
                    .strip("央视网消息（新闻联播）：")
                    .strip("(新闻联播)：")
                    .strip()
                    .replace("\n", " ")
                )
            except:  # noqa: E722
                continue
        temp_df = pd.DataFrame(
            data=[[date] * len(title_list), title_list, content_list],
            index=["date", "title", "content"],
        ).T
        return temp_df
    elif int(date) > int("20160203"):
        url = f"https://tv.cctv.com/lm/xwlb/day/{date}.shtml"
        r = requests.get(url)
        r.encoding = "utf-8"
        soup = BeautifulSoup(r.text, "lxml")
        page_url = [item.find("a")["href"] for item in soup.find_all("li")[1:]]
        title_list = []
        content_list = []
        headers = {
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,"
            "image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "Accept-Encoding": "gzip, deflate",
            "Accept-Language": "zh-CN,zh;q=0.9,en;q=0.8",
            "Cache-Control": "no-cache",
            "Cookie": "cna=DLYSGBDthG4CAbRVCNxSxGT6",
            "Host": "tv.cctv.com",
            "Pragma": "no-cache",
            "Proxy-Connection": "keep-alive",
            "Upgrade-Insecure-Requests": "1",
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) "
            "Chrome/92.0.4515.159 Safari/537.36",
        }
        for page in tqdm(page_url, leave=False):
            try:
                r = requests.get(page, headers=headers)
                r.encoding = "utf-8"
                soup = BeautifulSoup(r.text, features="lxml")
                if soup.find("h3"):
                    title = soup.find("h3").text
                else:
                    title = soup.find(name="div", attrs={"class": "tit"}).text
                if soup.find(name="div", attrs={"class": "cnt_bd"}):
                    content = soup.find(name="div", attrs={"class": "cnt_bd"}).text
                else:
                    content = soup.find(
                        name="div", attrs={"class": "content_area"}
                    ).text
                title_list.append(title.strip("[视频]").strip().replace("\n", " "))
                content_list.append(
                    content.strip()
                    .strip("央视网消息(新闻联播)：")
                    .strip("央视网消息（新闻联播）：")
                    .strip("(新闻联播)：")
                    .strip()
                    .replace("\n", " ")
                )
            except:  # noqa: E722
                continue
        temp_df = pd.DataFrame(
            data=[[date] * len(title_list), title_list, content_list],
            index=["date", "title", "content"],
        ).T
        return temp_df


if __name__ == "__main__":
    news_cctv_df = news_cctv(date="20240424")
    print(news_cctv_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2023/10/19 16:00
Desc: 50 ETF 期权波动率指数 QVIX
300 ETF 期权波动率指数 QVIX
http://1.optbbs.com/s/vix.shtml?50ETF
http://1.optbbs.com/s/vix.shtml?300ETF
"""

import pandas as pd
from functools import lru_cache


@lru_cache
def __get_optbbs_daily() -> pd.DataFrame:
    """
    读取原始数据
    http://1.optbbs.com/d/csv/d/k.csv
    :return: 原始数据
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/k.csv"
    temp_df = pd.read_csv(url, encoding="gbk")
    return temp_df


def index_option_50etf_qvix() -> pd.DataFrame:
    """
    50ETF 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?50ETF
    :return: 50ETF 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, :5]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_50etf_min_qvix() -> pd.DataFrame:
    """
    50 ETF 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?50ETF
    :return: 50 ETF 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vix50.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


def index_option_300etf_qvix() -> pd.DataFrame:
    """
    300 ETF 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?300ETF
    :return: 300 ETF 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, [0, 9, 10, 11, 12]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_300etf_min_qvix() -> pd.DataFrame:
    """
    300 ETF 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?300ETF
    :return: 300 ETF 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vix300.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


def index_option_500etf_qvix() -> pd.DataFrame:
    """
    500 ETF 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?500ETF
    :return: 500 ETF 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, [0, 67, 68, 69, 70]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_500etf_min_qvix() -> pd.DataFrame:
    """
    500 ETF 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?500ETF
    :return: 500 ETF 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vix500.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


def index_option_cyb_qvix() -> pd.DataFrame:
    """
    创业板 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?CYB
    :return: 创业板 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, [0, 71, 72, 73, 74]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_cyb_min_qvix() -> pd.DataFrame:
    """
    创业板 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?CYB
    :return: 创业板 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vixcyb.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


def index_option_kcb_qvix() -> pd.DataFrame:
    """
    科创板 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?KCB
    :return: 科创板 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, [0, 83, 84, 85, 86]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_kcb_min_qvix() -> pd.DataFrame:
    """
    科创板 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?KCB
    :return: 科创板 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vixkcb.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


def index_option_100etf_qvix() -> pd.DataFrame:
    """
    深证100ETF 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?100ETF
    :return: 深证100ETF 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, [0, 75, 76, 77, 78]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_100etf_min_qvix() -> pd.DataFrame:
    """
    深证100ETF 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?100ETF
    :return: 深证100ETF 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vix100.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


def index_option_300index_qvix() -> pd.DataFrame:
    """
    中证300股指 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?Index
    :return: 中证300股指 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, [0, 17, 18, 19, 20]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_300index_min_qvix() -> pd.DataFrame:
    """
    中证300股指 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?Index
    :return: 中证300股指 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vixindex.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


def index_option_1000index_qvix() -> pd.DataFrame:
    """
    中证1000股指 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?Index1000
    :return: 中证1000股指 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, [0, 25, 26, 27, 28]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_1000index_min_qvix() -> pd.DataFrame:
    """
    中证1000股指 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?Index1000
    :return: 中证1000股指 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vixindex1000.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


def index_option_50index_qvix() -> pd.DataFrame:
    """
    上证50股指 期权波动率指数 QVIX
    http://1.optbbs.com/s/vix.shtml?50index
    :return: 上证50股指 期权波动率指数 QVIX
    :rtype: pandas.DataFrame
    """
    temp_df = __get_optbbs_daily().iloc[:, [0, 79, 80, 81, 82]]
    temp_df.columns = [
        "date",
        "open",
        "high",
        "low",
        "close",
    ]
    temp_df.loc[:, "date"] = pd.to_datetime(temp_df["date"], errors="coerce").dt.date
    temp_df.loc[:, "open"] = pd.to_numeric(temp_df["open"], errors="coerce")
    temp_df.loc[:, "high"] = pd.to_numeric(temp_df["high"], errors="coerce")
    temp_df.loc[:, "low"] = pd.to_numeric(temp_df["low"], errors="coerce")
    temp_df.loc[:, "close"] = pd.to_numeric(temp_df["close"], errors="coerce")
    return temp_df


def index_option_50index_min_qvix() -> pd.DataFrame:
    """
    上证50股指 期权波动率指数 QVIX-分时
    http://1.optbbs.com/s/vix.shtml?50index
    :return: 上证50股指 期权波动率指数 QVIX-分时
    :rtype: pandas.DataFrame
    """
    url = "http://1.optbbs.com/d/csv/d/vix50index.csv"
    temp_df = pd.read_csv(url).iloc[:, :2]
    temp_df.columns = [
        "time",
        "qvix",
    ]
    temp_df.loc[:, "qvix"] = pd.to_numeric(temp_df["qvix"], errors="coerce")
    return temp_df


if __name__ == "__main__":
    index_option_50etf_qvix_df = index_option_50etf_qvix()
    print(index_option_50etf_qvix_df)

    index_option_50etf_min_qvix_df = index_option_50etf_min_qvix()
    print(index_option_50etf_min_qvix_df)

    index_option_300etf_qvix_df = index_option_300etf_qvix()
    print(index_option_300etf_qvix_df)

    index_option_300etf_min_qvix_df = index_option_300etf_min_qvix()
    print(index_option_300etf_min_qvix_df)

    index_option_500etf_qvix_df = index_option_500etf_qvix()
    print(index_option_500etf_qvix_df)

    index_option_500etf_min_qvix_df = index_option_500etf_min_qvix()
    print(index_option_500etf_min_qvix_df)

    index_option_cyb_qvix_df = index_option_cyb_qvix()
    print(index_option_cyb_qvix_df)

    index_option_cyb_min_qvix_df = index_option_cyb_min_qvix()
    print(index_option_cyb_min_qvix_df)

    index_option_kcb_qvix_df = index_option_kcb_qvix()
    print(index_option_kcb_qvix_df)

    index_option_kcb_min_qvix_df = index_option_kcb_min_qvix()
    print(index_option_kcb_min_qvix_df)

    index_option_100etf_qvix_df = index_option_100etf_qvix()
    print(index_option_100etf_qvix_df)

    index_option_100etf_min_qvix_df = index_option_100etf_min_qvix()
    print(index_option_100etf_min_qvix_df)

    index_option_300index_qvix_df = index_option_300index_qvix()
    print(index_option_300index_qvix_df)

    index_option_300index_min_qvix_df = index_option_300index_min_qvix()
    print(index_option_300index_min_qvix_df)

    index_option_1000index_qvix_df = index_option_1000index_qvix()
    print(index_option_1000index_qvix_df)

    index_option_1000index_min_qvix_df = index_option_1000index_min_qvix()
    print(index_option_1000index_min_qvix_df)

    index_option_50index_qvix_df = index_option_50index_qvix()
    print(index_option_50index_qvix_df)

    index_option_50index_min_qvix_df = index_option_50index_min_qvix()
    print(index_option_50index_min_qvix_df)



# -*- coding:utf-8 -*-
# !/usr/bin/env python
"""
Date: 2025/9/13 16:00
Desc: 深圳证券交易所-期权子网-行情数据-当日合约
"""
from io import BytesIO

import pandas as pd
import requests


def option_current_day_szse() -> pd.DataFrame:
    """
    深圳证券交易所-期权子网-行情数据-当日合约
    https://www.sse.org.cn/option/quotation/contract/daycontract/index.html
    :return: 深圳期权当日合约
    :rtype: pandas.DataFrame
    """
    import warnings
    warnings.filterwarnings(action='ignore', message="Workbook contains no default style")
    url = "https://www.sse.org.cn/api/report/ShowReport"
    params = {
        "SHOWTYPE": "xlsx",
        "CATALOGID": "option_drhy",
        "TABKEY": "tab1",
    }
    r = requests.get(url, params=params)
    temp_df = pd.read_excel(BytesIO(r.content))
    temp_df['序号'] = pd.to_numeric(temp_df['序号'], errors='coerce')
    temp_df['行权价'] = pd.to_numeric(temp_df['行权价'], errors='coerce')
    temp_df['合约单位'] = pd.to_numeric(temp_df['合约单位'], errors='coerce')
    temp_df['涨停价格'] = pd.to_numeric(temp_df['涨停价格'], errors='coerce')
    temp_df['跌停价格'] = pd.to_numeric(temp_df['跌停价格'], errors='coerce')
    temp_df['前结算价'] = pd.to_numeric(temp_df['前结算价'], errors='coerce')
    temp_df['合约总持仓'] = pd.to_numeric(temp_df['合约总持仓'], errors='coerce')
    temp_df['原行权价格'] = pd.to_numeric(temp_df['原行权价格'], errors='coerce')
    temp_df['原合约单位'] = pd.to_numeric(temp_df['原合约单位'], errors='coerce')
    temp_df['合约到期剩余交易天数'] = pd.to_numeric(temp_df['合约到期剩余交易天数'], errors='coerce')
    temp_df['合约到期剩余自然天数'] = pd.to_numeric(temp_df['合约到期剩余自然天数'], errors='coerce')
    temp_df['下次合约调整剩余交易天数'] = pd.to_numeric(temp_df['下次合约调整剩余交易天数'], errors='coerce')
    temp_df['下次合约调整剩余自然天数'] = pd.to_numeric(temp_df['下次合约调整剩余自然天数'], errors='coerce')
    temp_df['交易日期'] = pd.to_datetime(temp_df['交易日期'], errors='coerce').dt.date
    temp_df['最后交易日'] = pd.to_datetime(temp_df['最后交易日'], errors='coerce').dt.date
    temp_df['行权日'] = pd.to_datetime(temp_df['行权日'], errors='coerce').dt.date
    temp_df['到期日'] = pd.to_datetime(temp_df['到期日'], errors='coerce').dt.date
    temp_df['交收日'] = pd.to_datetime(temp_df['交收日'], errors='coerce').dt.date
    temp_df = temp_df[[
        '序号',
        '合约编码',
        '合约代码',
        '合约简称',
        '标的证券简称(代码)',
        '合约类型',
        '行权价',
        '合约单位',
        '最后交易日',
        '行权日',
        '到期日',
        '交收日',
        '新挂',
        '涨停价格',
        '跌停价格',
        '前结算价',
        '合约调整',
        '停牌',
        '合约总持仓',
        '挂牌原因',
        '原合约代码',
        '原合约简称',
        '原行权价格',
        '原合约单位',
        '合约到期剩余交易天数',
        '合约到期剩余自然天数',
        '下次合约调整剩余交易天数',
        '下次合约调整剩余自然天数',
        '交易日期',
    ]]
    return temp_df


if __name__ == "__main__":
    option_current_day_szse_df = option_current_day_szse()
    print(option_current_day_szse_df)
